@@ -31,11 +31,50 @@ class Error(Exception):
     pass
 
 
+#
+# Custom argparse.HelpFormatter classes.
+#
+
+class NoMetavarArgHelpFormatter(argparse.HelpFormatter):
+    def _format_action_invocation(self, action):
+        if not action.option_strings:
+            default = self._get_default_metavar_for_positional(action)
+            metavar, = self._metavar_formatter(action, default)(1)
+            return metavar
+
+        else:
+            parts = []
+            parts.extend(action.option_strings)
+            return ', '.join(parts)
+
+
+class NoMetavarArgRawTextHelpFormatter(argparse.RawTextHelpFormatter, NoMetavarArgHelpFormatter):
+    pass
+
+
+class NoMetavarUsageHelpFormatter(argparse.HelpFormatter):
+    def _format_args(self, action, default_metavar):
+        return ""
+    
+    
+class NoMetavarHelpFormatter(NoMetavarArgHelpFormatter, NoMetavarUsageHelpFormatter):
+    pass
+
+
+class NoMetavarRawTextHelpFormatter(argparse.RawTextHelpFormatter, NoMetavarHelpFormatter):
+    pass
+
+
+# This gets called when -f is passed on the command line. 
+
 class ParserScrobbleFormatAction(argparse.Action):
     def __call__(self, parser, namespace, values, option_string = None):
         namespace.timestamp = int(datetime.strptime(namespace.timestamp, values).timestamp())
         del namespace.format
 
+#
+# Constants.
+#
 
 API_KEY     = "b3e7abc138f65a43803f887aeb36b9f6"
 SECRET      = "d60a1a4d704b71c0e8e5bac98d793969"
@@ -44,6 +83,9 @@ dirs = AppDirs("scrobbler", "hauzer", "1.0.0")
 DB_FILE     = os.path.join(dirs.user_data_dir, "sessions.db")
 LFM_FILE    = os.path.join(dirs.user_data_dir, "lfm.dat")
 
+#
+# Repetitive code grouped into functions.
+#
 
 def user_exists(dbc, user):
     dbc.execute("select exists(select * from sessions where user == ?)", (user,))
@@ -60,6 +102,11 @@ def auth(app, dbc, user):
                     "Add a session via \"session-add\" first.".format(args.user))
     
 
+#
+# These get called when a corresponding sub-command on the command-line
+# is passed.
+#
+
 def cmd_session_add(app, dbc, args):
     errormsg = "Could not add user \"{}\" to the database; already exists."
     
@@ -147,68 +194,88 @@ def cmd_unp(app, dbc, args):
                                  albumartist = args.albumartist, context = args.context)
 
 
+#
+# Arguments parser building.
+#
 
-parser = argparse.ArgumentParser(description = "A Last.fm scrobbler and a now-playing status updater.")
-subparsers              = parser.add_subparsers()
-parser_cmd_session_add  = subparsers.add_parser("session-add", aliases = ["sa"])
-parser_cmd_session_list = subparsers.add_parser("session-list", aliases = ["sl"])
-parser_cmd_session_rm   = subparsers.add_parser("session-remove", aliases = ["sr"])
-parser_cmd_scrobble     = subparsers.add_parser("scrobble", aliases = ["sc"],
-                                            formatter_class = argparse.RawTextHelpFormatter)
-parser_cmd_unp          = subparsers.add_parser("update-now-playing", aliases = ["unp"])
+parser = argparse.ArgumentParser(description = "A Last.fm scrobbler and a now-playing status updater.",
+                                 formatter_class = argparse.RawTextHelpFormatter)
+
+subparsers              = parser.add_subparsers(metavar = "subcommand", title = "subcommands",
+                                                help =  "{0}session-add, sa\n" \
+                                                        "{0}session-list, sl\n" \
+                                                        "{0}session-remove, sr\n" \
+                                                        "{0}scrobble, sc\n" \
+                                                        "{0}update-now-playing, unp".format("\b" * 12))
 
 
-parser_cmd_session_add.add_argument("-u", "--user", metavar = "user")
+parser_cmd_session_add  = subparsers.add_parser("session-add", aliases = ["sa"],
+                                                formatter_class = NoMetavarHelpFormatter)
+parser_cmd_session_add.add_argument("-u", "--user")
 group = parser_cmd_session_add.add_mutually_exclusive_group()
-group.add_argument("-p", "--password", metavar = "pwd", dest = "pwd")
-group.add_argument("-s", "--session-key", metavar = "sk", dest = "sk")
+group.add_argument("-p", "--password", dest = "pwd")
+group.add_argument("-s", "--session-key", dest = "sk")
 parser_cmd_session_add.set_defaults(func = cmd_session_add)
 
+
+parser_cmd_session_list = subparsers.add_parser("session-list", aliases = ["sl"],
+                                                formatter_class = NoMetavarHelpFormatter)
 parser_cmd_session_list.set_defaults(func = cmd_session_list)
 
+
+parser_cmd_session_rm   = subparsers.add_parser("session-remove", aliases = ["sr"],
+                                                formatter_class = NoMetavarHelpFormatter)
 parser_cmd_session_rm.add_argument("user")
 parser_cmd_session_rm.set_defaults(func = cmd_session_rm)
 
 
-parser_scrobble = argparse.ArgumentParser(usage =  "A scrobble consists of three or more\n"    \
-                                                    "options specified below. Pass these "      \
-                                                    "quoted,\nand as you would to a program.",
-                                           add_help = False, formatter_class = argparse.RawTextHelpFormatter)
-parser_scrobble.add_argument("artist", metavar = "artist")
-parser_scrobble.add_argument("track", metavar = "track")
-parser_scrobble.add_argument("timestamp", metavar = "timestamp",
-                              help = "\n{0}Time of scrobbling. Can be formatted with -f,\n{0}otherwise it's " \
-                              "an UNIX timestamp.".format("\b" * 18))
-parser_scrobble.add_argument("-f", "--format", metavar = "format", action = ParserScrobbleFormatAction,
-                              help = "{0}Specifies the format of the timestamp. Uses\n{0}the same syntax " \
-                              "as Python's strftime().".format("\b" * 18))
-parser_scrobble.add_argument("-a", "--album", metavar = "album")
-parser_scrobble.add_argument("-d", "--duration", metavar = "duration")
-parser_scrobble.add_argument("-m", "--mbid", metavar = "mbid")
-parser_scrobble.add_argument("-t", "--track-number", metavar = "track_number", dest = "tracknumber")
-parser_scrobble.add_argument("-aa", "--album-artist", metavar = "album_artist", dest = "albumartist")
-parser_scrobble.add_argument("-s", "--stream-id", metavar = "stream_id", dest = "streamid")
+parser_scrobble = argparse.ArgumentParser(usage =   "A scrobble consists of three or more\n"        \
+                                                    "{0}options. Pass these quoted, and as you\n"   \
+                                                    "{0}would to a program.".format(" " * 7),
+                                           add_help = False, formatter_class = NoMetavarRawTextHelpFormatter)
+parser_scrobble.add_argument("artist")
+parser_scrobble.add_argument("track")
+parser_scrobble.add_argument("timestamp",
+                             help = "\n{0}Time of scrobbling. Can be formatted with -f,\n{0}otherwise it's " \
+                                    "an UNIX timestamp.".format("\b" * 17))
+parser_scrobble.add_argument("-f", "--format", action = ParserScrobbleFormatAction,
+                              help = "\n{0}Specifies the format of the timestamp.\n{0}Uses the same syntax " \
+                              "as Python's strftime().\n\n".format("\b" * 14))
+parser_scrobble.add_argument("-a", "--album")
+parser_scrobble.add_argument("-d", "--duration")
+parser_scrobble.add_argument("-m", "--mbid")
+parser_scrobble.add_argument("-t", "--track-number", dest = "tracknumber")
+parser_scrobble.add_argument("-aa", "--album-artist", dest = "albumartist")
+parser_scrobble.add_argument("-s", "--stream-id", dest = "streamid")
 parser_scrobble.add_argument("-c", "--chosen-by-user", action = "store_true", dest = "chosenbyuser")
-parser_scrobble.add_argument("-cx", "--context", metavar = "context")
+parser_scrobble.add_argument("-cx", "--context")
 
+
+parser_cmd_scrobble     = subparsers.add_parser("scrobble", aliases = ["sc"],
+                                                formatter_class = NoMetavarArgRawTextHelpFormatter)
 parser_cmd_scrobble.add_argument("user")
 parser_cmd_scrobble.add_argument("-s", "--scrobble", action = "append", metavar = "\"artist track tstamp ...\"",
                              dest = "scrobbles", required = True, help = parser_scrobble.format_help())
 parser_cmd_scrobble.set_defaults(func = cmd_scrobble)
 
 
+parser_cmd_unp          = subparsers.add_parser("update-now-playing", aliases = ["unp"],
+                                                formatter_class = NoMetavarHelpFormatter)
 parser_cmd_unp.add_argument("user")
-parser_cmd_unp.add_argument("artist", metavar = "artist")
-parser_cmd_unp.add_argument("track", metavar = "track")
-parser_cmd_unp.add_argument("-a", "--album", metavar = "album")
-parser_cmd_unp.add_argument("-d", "--duration", metavar = "duration")
-parser_cmd_unp.add_argument("-m", "--mbid", metavar = "mbid")
-parser_cmd_unp.add_argument("-t", "--track-number", metavar = "track_number", dest = "tracknumber")
-parser_cmd_unp.add_argument("-aa", "--album-artist", metavar = "album_artist", dest = "albumartist")
-parser_cmd_unp.add_argument("-cx", "--context", metavar = "context")
+parser_cmd_unp.add_argument("artist")
+parser_cmd_unp.add_argument("track")
+parser_cmd_unp.add_argument("-a", "--album")
+parser_cmd_unp.add_argument("-d", "--duration")
+parser_cmd_unp.add_argument("-m", "--mbid")
+parser_cmd_unp.add_argument("-t", "--track-number", dest = "tracknumber")
+parser_cmd_unp.add_argument("-aa", "--album-artist", dest = "albumartist")
+parser_cmd_unp.add_argument("-cx", "--context")
 parser_cmd_unp.set_defaults(func = cmd_unp)
 
 
+#
+# Begin the actual execution of the script.
+#
 
 args = parser.parse_args()
 os.makedirs(dirs.user_data_dir, exist_ok = True)
@@ -223,9 +290,14 @@ if not dbc.fetchone()[0]:
     dbc.execute("create table sessions (user text primary key, key text)")
 
 try:
-    args.func(app, dbc, args)
-except Error as err:
-    print(err)
+    args.func
+except AttributeError:
+    parser.print_help()
+else:
+    try:
+        args.func(app, dbc, args)
+    except Error as err:
+        print(err)
 
 db.commit()
 dbc.close()
